"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamoDBStreamEnvelope = void 0;
const index_js_1 = require("../schemas/index.js");
const envelope_js_1 = require("./envelope.js");
const errors_js_1 = require("../errors.js");
/**
 * DynamoDB Stream Envelope to extract data within NewImage/OldImage
 *
 * Note: Values are the parsed models. Images' values can also be None, and
 * length of the list is the record's amount in the original event.
 */
class DynamoDBStreamEnvelope extends envelope_js_1.Envelope {
    static parse(data, schema) {
        const parsedEnvelope = index_js_1.DynamoDBStreamSchema.parse(data);
        return parsedEnvelope.Records.map((record) => {
            return {
                NewImage: super.parse(record.dynamodb.NewImage, schema),
                OldImage: super.parse(record.dynamodb.OldImage, schema),
            };
        });
    }
    static safeParse(data, schema) {
        const parsedEnvelope = index_js_1.DynamoDBStreamSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new errors_js_1.ParseError('Failed to parse DynamoDB Stream envelope', {
                    cause: parsedEnvelope.error,
                }),
                originalEvent: data,
            };
        }
        const parsedLogEvents = [];
        for (const record of parsedEnvelope.data.Records) {
            const parsedNewImage = super.safeParse(record.dynamodb.NewImage, schema);
            const parsedOldImage = super.safeParse(record.dynamodb.OldImage, schema);
            if (!parsedNewImage.success || !parsedOldImage.success) {
                return {
                    success: false,
                    error: !parsedNewImage.success
                        ? new errors_js_1.ParseError('Failed to parse NewImage', {
                            cause: parsedNewImage.error,
                        })
                        : new errors_js_1.ParseError('Failed to parse OldImage', {
                            cause: parsedOldImage.error,
                        }),
                    originalEvent: data,
                };
            }
            else {
                parsedLogEvents.push({
                    NewImage: parsedNewImage.data,
                    OldImage: parsedOldImage.data,
                });
            }
        }
        return {
            success: true,
            data: parsedLogEvents,
        };
    }
}
exports.DynamoDBStreamEnvelope = DynamoDBStreamEnvelope;
