"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KinesisEnvelope = void 0;
const envelope_js_1 = require("./envelope.js");
const kinesis_js_1 = require("../schemas/kinesis.js");
const errors_js_1 = require("../errors.js");
/**
 * Kinesis Data Stream Envelope to extract array of Records
 *
 * The record's data parameter is a base64 encoded string which is parsed into a bytes array,
 * though it can also be a JSON encoded string.
 * Regardless of its type it'll be parsed into a BaseModel object.
 *
 * Note: Records will be parsed the same way so if model is str,
 * all items in the list will be parsed as str and not as JSON (and vice versa)
 */
class KinesisEnvelope extends envelope_js_1.Envelope {
    static parse(data, schema) {
        const parsedEnvelope = kinesis_js_1.KinesisDataStreamSchema.parse(data);
        return parsedEnvelope.Records.map((record) => {
            return super.parse(record.kinesis.data, schema);
        });
    }
    static safeParse(data, schema) {
        const parsedEnvelope = kinesis_js_1.KinesisDataStreamSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new errors_js_1.ParseError('Failed to parse Kinesis Data Stream envelope', {
                    cause: parsedEnvelope.error,
                }),
                originalEvent: data,
            };
        }
        const parsedRecords = [];
        for (const record of parsedEnvelope.data.Records) {
            const parsedRecord = super.safeParse(record.kinesis.data, schema);
            if (!parsedRecord.success) {
                return {
                    success: false,
                    error: new errors_js_1.ParseError('Failed to parse Kinesis Data Stream record', {
                        cause: parsedRecord.error,
                    }),
                    originalEvent: data,
                };
            }
            parsedRecords.push(parsedRecord.data);
        }
        return {
            success: true,
            data: parsedRecords,
        };
    }
}
exports.KinesisEnvelope = KinesisEnvelope;
