"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SnsSqsEnvelope = exports.SnsEnvelope = void 0;
const envelope_js_1 = require("./envelope.js");
const sns_js_1 = require("../schemas/sns.js");
const sqs_js_1 = require("../schemas/sqs.js");
const errors_js_1 = require("../errors.js");
/**
 * SNS Envelope to extract array of Records
 *
 * The record's body parameter is a string, though it can also be a JSON encoded string.
 * Regardless of its type it'll be parsed into a BaseModel object.
 *
 * Note: Records will be parsed the same way so if model is str,
 * all items in the list will be parsed as str and npt as JSON (and vice versa)
 */
class SnsEnvelope extends envelope_js_1.Envelope {
    static parse(data, schema) {
        const parsedEnvelope = sns_js_1.SnsSchema.parse(data);
        return parsedEnvelope.Records.map((record) => {
            return super.parse(record.Sns.Message, schema);
        });
    }
    static safeParse(data, schema) {
        const parsedEnvelope = sns_js_1.SnsSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new errors_js_1.ParseError(`Failed to parse SNS envelope`, {
                    cause: parsedEnvelope.error,
                }),
                originalEvent: data,
            };
        }
        const parsedMessages = [];
        for (const record of parsedEnvelope.data.Records) {
            const parsedMessage = super.safeParse(record.Sns.Message, schema);
            if (!parsedMessage.success) {
                return {
                    success: false,
                    error: new errors_js_1.ParseError(`Failed to parse SNS message`, {
                        cause: parsedMessage.error,
                    }),
                    originalEvent: data,
                };
            }
            parsedMessages.push(parsedMessage.data);
        }
        return {
            success: true,
            data: parsedMessages,
        };
    }
}
exports.SnsEnvelope = SnsEnvelope;
/**
 *  SNS plus SQS Envelope to extract array of Records
 *
 *  Published messages from SNS to SQS has a slightly different payload.
 *  Since SNS payload is marshalled into `Record` key in SQS, we have to:
 *
 *  1. Parse SQS schema with incoming data
 *  2. Unmarshall SNS payload and parse against SNS Notification schema not SNS/SNS Record
 *  3. Finally, parse provided model against payload extracted
 *
 */
class SnsSqsEnvelope extends envelope_js_1.Envelope {
    static parse(data, schema) {
        const parsedEnvelope = sqs_js_1.SqsSchema.parse(data);
        return parsedEnvelope.Records.map((record) => {
            const snsNotification = sns_js_1.SnsSqsNotificationSchema.parse(JSON.parse(record.body));
            return super.parse(snsNotification.Message, schema);
        });
    }
    static safeParse(data, schema) {
        const parsedEnvelope = sqs_js_1.SqsSchema.safeParse(data);
        if (!parsedEnvelope.success) {
            return {
                success: false,
                error: new errors_js_1.ParseError(`Failed to parse SQS envelope`, {
                    cause: parsedEnvelope.error,
                }),
                originalEvent: data,
            };
        }
        const parsedMessages = [];
        // JSON.parse can throw an error, thus we catch it and return ParsedErrorResult
        try {
            for (const record of parsedEnvelope.data.Records) {
                const snsNotification = sns_js_1.SnsSqsNotificationSchema.safeParse(JSON.parse(record.body));
                if (!snsNotification.success) {
                    return {
                        success: false,
                        error: new errors_js_1.ParseError(`Failed to parse SNS notification`, {
                            cause: snsNotification.error,
                        }),
                        originalEvent: data,
                    };
                }
                const parsedMessage = super.safeParse(snsNotification.data.Message, schema);
                if (!parsedMessage.success) {
                    return {
                        success: false,
                        error: new errors_js_1.ParseError(`Failed to parse SNS message`, {
                            cause: parsedMessage.error,
                        }),
                        originalEvent: data,
                    };
                }
                parsedMessages.push(parsedMessage.data);
            }
        }
        catch (e) {
            return {
                success: false,
                error: e,
                originalEvent: data,
            };
        }
        return { success: true, data: parsedMessages };
    }
}
exports.SnsSqsEnvelope = SnsSqsEnvelope;
